--[================[
LibClassicCasterino
Author: d87
--]================]


local MAJOR, MINOR = "LibClassicCasterino", 7
local lib = LibStub:NewLibrary(MAJOR, MINOR)
if not lib then return end


lib.callbacks = lib.callbacks or LibStub("CallbackHandler-1.0"):New(lib)

lib.frame = lib.frame or CreateFrame("Frame")

local f = lib.frame
local callbacks = lib.callbacks

lib.casters = lib.casters or {} -- setmetatable({}, { __mode = "v" })
local casters = lib.casters

-- local guidsToPurge = {}

local UnitGUID = UnitGUID
local bit_band = bit.band
local GetTime = GetTime
local CastingInfo = CastingInfo
local ChannelInfo = ChannelInfo

local COMBATLOG_OBJECT_TYPE_PLAYER = COMBATLOG_OBJECT_TYPE_PLAYER
local classCasts
local classChannels
local talentDecreased
local FireToUnits

f:SetScript("OnEvent", function(self, event, ...)
    return self[event](self, event, ...)
end)


local spellNameToID = {}
local NPCspellNameToID = {}
local NPCSpells

local castTimeCache = {}
local castTimeCacheStartTimes = setmetatable({}, { __mode = "v" })

--[[
function DUMPCASTS()
    local castedSpells = {}
    local counter = 0
    for id=1,40000 do
        local name, _, texture, castTime = GetSpellInfo(id)
        136235 -- interface/icons/temp.blp -- Samwise Didier Icon aka missing icon
        if name and castTime > 500 and texture ~= 136235 then
            castedSpells[id] = true
            counter = counter + 1
        end
    end
    print(counter)
    NugHealthDB.LCDDUMP = castedSpells
end
]]

local refreshCastTable = function(tbl, ...)
    local numArgs = select("#", ...)
    for i=1, numArgs do
        tbl[i] = select(i, ...)
    end
end

local makeCastUID = function(guid, spellName)
    local _, _, _, _, _, npcID = strsplit("-", guid);
    return npcID..spellName
end

local function CastStart(srcGUID, castType, spellName, spellID, overrideCastTime )
    local _, _, icon, castTime = GetSpellInfo(spellID)
    if castType == "CHANNEL" then
        castTime = classChannels[spellID]*1000
        local decreased = talentDecreased[spellID]
        if decreased then
            castTime = castTime - decreased
        end
    end
    if overrideCastTime then
        castTime = overrideCastTime
    end
    local now = GetTime()*1000
    local startTime = now
    local endTime = now + castTime
    local currentCast = casters[srcGUID]

    if currentCast then
        refreshCastTable(currentCast, castType, spellName, icon, startTime, endTime, spellID )
    else
        casters[srcGUID] = { castType, spellName, icon, startTime, endTime, spellID }
    end


    if castType == "CAST" then
        FireToUnits("UNIT_SPELLCAST_START", srcGUID)
    else
        FireToUnits("UNIT_SPELLCAST_CHANNEL_START", srcGUID)
    end
end

local function CastStop(srcGUID, castType, suffix )
    local currentCast = casters[srcGUID]
    if currentCast then
        castType = castType or currentCast[1]

        casters[srcGUID] = nil

        if castType == "CAST" then
            local event = "UNIT_SPELLCAST_"..suffix
            FireToUnits(event, srcGUID)
        else
            FireToUnits("UNIT_SPELLCAST_CHANNEL_STOP", srcGUID)
        end
    end
end

function f:COMBAT_LOG_EVENT_UNFILTERED(event)

    local timestamp, eventType, hideCaster,
    srcGUID, srcName, srcFlags, srcFlags2,
    dstGUID, dstName, dstFlags, dstFlags2,
    spellID, spellName, arg3, arg4, arg5 = CombatLogGetCurrentEventInfo()

    local isSrcPlayer = bit_band(srcFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0
    if isSrcPlayer and spellID == 0 then
        spellID = spellNameToID[spellName]
    end
    if eventType == "SPELL_CAST_START" then
        if isSrcPlayer then
            local isCasting = classCasts[spellID]
            if isCasting then
                CastStart(srcGUID, "CAST", spellName, spellID)
            end
        else
            local castUID = makeCastUID(srcGUID, spellName)
            local cachedTime = castTimeCache[castUID]
            local spellID = NPCspellNameToID[spellName] -- just for the icon
            if not spellID then
                spellID = 2050
            end
            if cachedTime then
                CastStart(srcGUID, "CAST", spellName, spellID, cachedTime*1000)
            else
                castTimeCacheStartTimes[srcGUID..castUID] = GetTime()
                CastStart(srcGUID, "CAST", spellName, spellID, 1500) -- using default 1.5s cast time for now
            end
        end
    elseif eventType == "SPELL_CAST_FAILED" then

            CastStop(srcGUID, "CAST", "FAILED")

    elseif eventType == "SPELL_CAST_SUCCESS" then
            if isSrcPlayer and classChannels[spellID] then
                -- SPELL_CAST_SUCCESS can come right after AURA_APPLIED, so ignoring it
                return
            end
            if not isSrcPlayer then
                local castUID = makeCastUID(srcGUID, spellName)
                local cachedTime = castTimeCache[castUID]
                if not cachedTime then
                    local restoredStartTime = castTimeCacheStartTimes[srcGUID..castUID]
                    if restoredStartTime then
                        local now = GetTime()
                        local castTime = now - restoredStartTime
                        if castTime < 10 then
                            castTimeCache[castUID] = castTime
                        end
                    end
                end
            end
            CastStop(srcGUID, nil, "STOP")

    elseif eventType == "SPELL_INTERRUPT" then

            CastStop(dstGUID, nil, "INTERRUPTED")

    elseif  eventType == "SPELL_AURA_APPLIED" or
            eventType == "SPELL_AURA_REFRESH" or
            eventType == "SPELL_AURA_APPLIED_DOSE"
    then
        if isSrcPlayer then
            local isChanneling = classChannels[spellID]
            if isChanneling then
                CastStart(srcGUID, "CHANNEL", spellName, spellID)
            end
        end
    elseif eventType == "SPELL_AURA_REMOVED" then
        if isSrcPlayer then
            local isChanneling = classChannels[spellID]
            if isChanneling then
                CastStop(srcGUID, "CHANNEL", "STOP")
            end
        end
    end

end

-- local castTimeIncreases = {
--     [1714] = 60,    -- Curse of Tongues (60%)
--     [5760] = 60,    -- Mind-Numbing Poison (60%)
-- }
local function IsSlowedDown(unit)
    for i=1,16 do
        local name, _, _, _, _, _, _, _, _, spellID = UnitAura(unit, i, "HARMFUL")
        if not name then return end
        if spellID == 1714 or spellID == 5760 then
            return true
        end
    end
end

function lib:UnitCastingInfo(unit)
    if unit == "player" then return CastingInfo() end
    local guid = UnitGUID(unit)
    local cast = casters[guid]
    if cast then
        local castType, name, icon, startTimeMS, endTimeMS, spellID = unpack(cast)
        if IsSlowedDown(unit) then
            local duration = endTimeMS - startTimeMS
            endTimeMS = startTimeMS + duration * 1.6
        end
        if castType == "CAST" and endTimeMS > GetTime()*1000 then
            local castID = nil
            return name, nil, icon, startTimeMS, endTimeMS, nil, castID, false, spellID
        end
    end
end

function lib:UnitChannelInfo(unit)
    if unit == "player" then return ChannelInfo() end
    local guid = UnitGUID(unit)
    local cast = casters[guid]
    if cast then
        local castType, name, icon, startTimeMS, endTimeMS, spellID = unpack(cast)
        -- Curse of Tongues doesn't matter that much for channels, skipping
        if castType == "CHANNEL" and endTimeMS > GetTime()*1000 then
            return name, nil, icon, startTimeMS, endTimeMS, nil, false, spellID
        end
    end
end


local Passthrough = function(self, event, unit)
    if unit == "player" then
        callbacks:Fire(event, unit)
    end
end
f.UNIT_SPELLCAST_START = Passthrough
f.UNIT_SPELLCAST_DELAYED = Passthrough
f.UNIT_SPELLCAST_STOP = Passthrough
f.UNIT_SPELLCAST_FAILED = Passthrough
f.UNIT_SPELLCAST_INTERRUPTED = Passthrough
f.UNIT_SPELLCAST_CHANNEL_START = Passthrough
f.UNIT_SPELLCAST_CHANNEL_UPDATE = Passthrough
f.UNIT_SPELLCAST_CHANNEL_STOP = Passthrough

function callbacks.OnUsed()
    f:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")

    f:RegisterEvent("UNIT_SPELLCAST_START")
    f:RegisterEvent("UNIT_SPELLCAST_DELAYED")
    f:RegisterEvent("UNIT_SPELLCAST_STOP")
    f:RegisterEvent("UNIT_SPELLCAST_FAILED")
    f:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
    f:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
    f:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
    f:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")

    -- for unit lookup
    f:RegisterEvent("GROUP_ROSTER_UPDATE")
    f:RegisterEvent("NAME_PLATE_UNIT_ADDED")
    f:RegisterEvent("NAME_PLATE_UNIT_REMOVED")
end

function callbacks.OnUnused()
    f:UnregisterAllEvents()
end

talentDecreased = {
    [25311] = 0.8,    -- Corruption (while leveling)
    [17924] = 2,       -- Soul Fire
    [25307] = 0.5,      -- Shadow Bolt
    [25309] = 0.5,      -- Immolate
    [691] = 4,        -- Summon Felhunter
    [688] = 4,        -- Summon Imp
    [697] = 4,        -- Summon Voidwalker
    [712] = 4,        -- Summon Succubus

    [15208] = 1,        -- Lightning Bolt
    [10605] = 1,        -- Chain Lightning
    [25357] = 0.5,      -- Healing Wave
    [2645] = 2,       -- Ghost Wolf

    [25304] = 0.5,      -- Frostbolt
    [25306] = 0.5,      -- Fireball


    [10934] = 0.5,      -- Smite
    [15261] = 0.5,    -- Holy Fire
    [6064] = 0.5,     -- Heal
    [25314] = 0.5,    -- Greater Heal
    [10876] = 0.5,     -- Mana Burn

    [9912] = 0.5,     -- Wrath
    [25298] = 0.5,     -- Starfire
    [25297] = 0.5,     -- Healing Touch
}

classCasts = {
    [25311] = 2, -- Corruption
    [6215] = 1.5, -- Fear
    [17928] = 2, -- Howl of Terror
    [18647] = 1.5, -- Banish
    [6366] = 3, -- Create Firestone (Lesser)
    [17951] = 3, -- Create Firestone
    [17952] = 3, -- Create Firestone (Greater)
    [17953] = 3, -- Create Firestone (Major)
    [28023] = 3, -- Create Healthstone
    [11729] = 3, -- Create Healthstone (Greater)
    [6202] = 3, -- Create Healthstone (Lesser)
    [11730] = 3, -- Create Healthstone (Major)
    [6201] = 3, -- Create Healthstone (Minor)
    [20755] = 3, -- Create Soulstone
    [20756] = 3, -- Create Soulstone (Greater)
    [20752] = 3, -- Create Soulstone (Lesser)
    [20757] = 3, -- Create Soulstone (Major)
    [693] = 3, -- Create Soulstone (Minor)
    [2362] = 5, -- Create Spellstone
    [17727] = 5, -- Create Spellstone (Greater)
    [17728] = 5, -- Create Spellstone (Major)
    [11726] = 3, -- Enslave Demon
    [126] = 5, -- Eye of Kilrogg
    [1122] = 2, -- Inferno
    [23161] = 3, -- Summon Dreadsteed
    [5784] = 3, -- Summon Felsteed
    [691] = 10, -- Summon Felhunter
    [688] = 10, -- Summon Imp
    [697] = 10, -- Summon Voidwalker
    [712] = 10, -- Summon Succubus
    [25309] = 2, -- Immolate
    [17923] = 1.5, -- Searing Pain
    [25307] = 3, -- Shadow Bolt
    [17924] = 4, -- Soul Fire

    [9853] = 1.5, -- Entangling Roots
    [18658] = 1.5, -- Hibernate
    [9901] = 1.5, -- Soothe Animal
    [25298] = 3.5, -- Starfire
    [18960] = 10, -- Teleport: Moonglade
    [9912] = 2, -- Wrath
    [25297] = 3.5, -- Healing Touch
    [20748] = 2, -- Rebirth
    [9858] = 2, -- Regrowth

    [28612] = 3, -- Conjure Food
    [759] = 3, -- Conjure Mana Agate
    [10053] = 3, -- Conjure Mana Citrine
    [3552] = 3, -- Conjure Mana Jade
    [10054] = 3, -- Conjure Mana Ruby
    [10140] = 3, -- Conjure Water
    [12826] = 1.5, -- Polymorph
    [28270] = 1.5, -- Polymorph: Cow
    [25306] = 3.5, -- Fireball
    [10216] = 3, -- Flamestrike
    [10207] = 1.5, -- Scorch
    [25304] = 3, -- Frostbolt

    [10876] = 3, -- Mana Burn
    [10955] = 1.5, -- Shackle Undead
    [10917] = 1.5, -- Flash Heal
    [25314] = 3, -- Greater Heal
    [6064] = 3, -- Heal
    [15261] = 3.5, -- Holy Fire
    [2053] = 2.5, -- Lesser Heal
    [25316] = 3, -- Prayer of Healing
    [20770] = 10, -- Resurrection
    [10934] = 2.5, -- Smite
    [10947] = 1.5, -- Mind Blast
    [10912] = 3, -- Mind Control

    [19943] = 1.5, -- Flash of Light
    [24239] = 1, -- Hammer of Wrath
    [25292] = 2.5, -- Holy Light
    [10318] = 2, -- Holy Wrath
    [20773] = 10, -- Redemption
    [23214] = 3, -- Summon Charger
    [13819] = 3, -- Summon Warhorse
    [10326] = 1.5, -- Turn Undead

    [10605] = 2.5, -- Chain Lightning
    [15208] = 3, -- Lightning Bolt
    [556] = 10, -- Astral Recall
    [6196] = 2, -- Far Sight
    [2645] = 3, -- Ghost Wolf
    [20777] = 10, -- Ancestral Spirit
    [10623] = 2.5, -- Chain Heal
    [25357] = 3, -- Healing Wave
    [10468] = 1.5, -- Lesser Healing Wave

    [1842] = 2, -- Disarm Trap
    -- missing poison creation

    [11605] = 1.5, -- Slam

    [20904] = 3, -- Aimed Shot
    [1002] = 2, -- Eyes of the Beast
    [2641] = 5, -- Dismiss pet
    [982] = 10, -- Revive Pet
    [14327] = 1.5, -- Scare Beast

    [8690] = 10, -- Hearthstone
    [4068] = 1, -- Iron Grenade

    -- Munts do not generate SPELL_CAST_START
    -- [8394] = 3, -- Striped Frostsaber
    -- [10793] = 3, -- Striped Nightsaber
}

classChannels = {
    -- [18807] = 3, -- Mind Flay

    [746] = 7,      -- First Aid
    [13278] = 4,    -- Gnomish Death Ray
    [20577] = 10,   -- Cannibalize
    [19305] = 6,    -- Starshards

    -- DRUID
    [17402] = 9.5,  -- Hurricane
    [9863] = 9.5,      -- Tranquility

    -- HUNTER
    [6197] = 60,     -- Eagle Eye
    [13544] = 5,     -- Mend Pet
    [1515] = 20,     -- Tame Beast
    [1002] = 60,     -- Eyes of the Beast
    [14295] = 6,     -- Volley

    -- MAGE
    [25345] = 5,     -- Arcane Missiles
    [10187] = 8,     -- Blizzard
    [12051] = 8,     -- Evocation

    -- PRIEST
    [18807] = 3,    -- Mind Flay
    [2096] = 60,    -- Mind Vision
    [10912] = 3,    -- Mind Control

    -- WARLOCK
    [126] = 45,       -- Eye of Kilrogg
    [11700] = 4.5,    -- Drain Life
    [11704] = 4.5,    -- Drain Mana
    [11675] = 14.5,   -- Drain Soul
    [11678] = 7.5,    -- Rain of Fire
    [11684] = 15,     -- Hellfire
    [11695] = 10,     -- Health Funnel
}

for id in pairs(classCasts) do
    spellNameToID[GetSpellInfo(id)] = id
    -- AddSpellNameRecognition(id)
end
for id in pairs(classChannels) do
    spellNameToID[GetSpellInfo(id)] = id
    -- AddSpellNameRecognition(id)
end

local partyGUIDtoUnit = {}
local raidGUIDtoUnit = {}
local nameplateUnits = {}
local commonUnits = {
    -- "player",
    "target",
    "targettarget",
    "pet",
}

function f:NAME_PLATE_UNIT_ADDED(event, unit)
    nameplateUnits[unit] = true
end


function f:NAME_PLATE_UNIT_REMOVED(event, unit)
    nameplateUnits[unit] = nil
end

function f:GROUP_ROSTER_UPDATE()
    table.wipe(partyGUIDtoUnit)
    table.wipe(raidGUIDtoUnit)
    if IsInGroup() then
        for i=1,4 do
            local unit = "party"..i
            local guid = UnitGUID(unit)
            if guid then
                partyGUIDtoUnit[guid] = unit
            end
        end
    end
    if IsInRaid() then
        for i=1,40 do
            local unit = "raid"..i
            local guid = UnitGUID(unit)
            if guid then
                raidGUIDtoUnit[guid] = unit
            end
        end
    end
end

FireToUnits = function(event, guid, ...)
    for _, unit in ipairs(commonUnits) do
        if UnitGUID(unit) == guid then
            callbacks:Fire(event, unit, ...)
        end
    end

    local partyUnit = partyGUIDtoUnit[guid]
    if partyUnit then
        callbacks:Fire(event, partyUnit, ...)
    end

    local raidUnit = raidGUIDtoUnit[guid]
    if raidUnit then
        callbacks:Fire(event, raidUnit, ...)
    end

    for unit in pairs(nameplateUnits) do
        if UnitGUID(unit) == guid then
            callbacks:Fire(event, unit, ...)
        end
    end
end


if lib.NPCSpellsTimer then
    lib.NPCSpellsTimer:Cancel()
end

local prevID
local counter = 0
local function processNPCSpellTable()
    counter = 0
    local id = next(NPCSpells, prevID)
    while (id and counter < 150) do
        NPCspellNameToID[GetSpellInfo(id)] = id

        counter = counter + 1
        prevID = id
        id = next(NPCSpells, prevID)
    end
    if (id) then
        C_Timer.After(1, processNPCSpellTable)
    end
end
lib.NPCSpellsTimer = C_Timer.NewTimer(6.5, processNPCSpellTable)

NPCSpells = {
    [10215] = true,
    [16587] = true,
    [16651] = true,
    [20874] = true,
    [16971] = true,
    [10695] = true,
    [7428] = true,
    [30152] = true,
    [7588] = true,
    [15238] = true,
    [11399] = true,
    [11431] = true,
    [7828] = true,
    [23242] = true,
    [7892] = true,
    [15910] = true,
    [8004] = true,
    [11975] = true,
    [16102] = true,
    [12039] = true,
    [12071] = true,
    [12167] = true,
    [20299] = true,
    [20427] = true,
    [513] = true,
    [16588] = true,
    [4165] = true,
    [20811] = true,
    [20875] = true,
    [25034] = true,
    [529] = true,
    [21067] = true,
    [21131] = true,
    [8552] = true,
    [8712] = true,
    [547] = true,
    [8776] = true,
    [555] = true,
    [8936] = true,
    [569] = true,
    [9224] = true,
    [4629] = true,
    [581] = true,
    [585] = true,
    [587] = true,
    [591] = true,
    [2371] = true,
    [13607] = true,
    [2387] = true,
    [2395] = true,
    [9672] = true,
    [27658] = true,
    [27722] = true,
    [13895] = true,
    [9928] = true,
    [4981] = true,
    [19980] = true,
    [14119] = true,
    [635] = true,
    [2547] = true,
    [639] = true,
    [10248] = true,
    [2579] = true,
    [647] = true,
    [5189] = true,
    [20876] = true,
    [16973] = true,
    [2667] = true,
    [17293] = true,
    [10792] = true,
    [17613] = true,
    [2739] = true,
    [11016] = true,
    [11048] = true,
    [693] = true,
    [15207] = true,
    [697] = true,
    [2795] = true,
    [5605] = true,
    [18445] = true,
    [707] = true,
    [2835] = true,
    [711] = true,
    [11400] = true,
    [22924] = true,
    [5781] = true,
    [23308] = true,
    [15783] = true,
    [735] = true,
    [27659] = true,
    [23628] = true,
    [19725] = true,
    [16071] = true,
    [19981] = true,
    [12072] = true,
    [28299] = true,
    [759] = true,
    [3067] = true,
    [3075] = true,
    [3083] = true,
    [3091] = true,
    [16590] = true,
    [6213] = true,
    [3115] = true,
    [8489] = true,
    [25292] = true,
    [8617] = true,
    [17294] = true,
    [8681] = true,
    [6405] = true,
    [6421] = true,
    [8809] = true,
    [6469] = true,
    [8873] = true,
    [8905] = true,
    [6517] = true,
    [3275] = true,
    [26444] = true,
    [3323] = true,
    [3331] = true,
    [837] = true,
    [6725] = true,
    [13480] = true,
    [6757] = true,
    [3387] = true,
    [9481] = true,
    [9513] = true,
    [13640] = true,
    [855] = true,
    [857] = true,
    [3443] = true,
    [6917] = true,
    [867] = true,
    [6949] = true,
    [3491] = true,
    [3507] = true,
    [9961] = true,
    [9993] = true,
    [24141] = true,
    [7077] = true,
    [3563] = true,
    [895] = true,
    [7221] = true,
    [16655] = true,
    [3635] = true,
    [25037] = true,
    [913] = true,
    [915] = true,
    [25357] = true,
    [17231] = true,
    [10697] = true,
    [10793] = true,
    [939] = true,
    [15048] = true,
    [943] = true,
    [11017] = true,
    [15208] = true,
    [26381] = true,
    [26445] = true,
    [11209] = true,
    [959] = true,
    [3843] = true,
    [3851] = true,
    [3859] = true,
    [22798] = true,
    [11433] = true,
    [18831] = true,
    [15592] = true,
    [7861] = true,
    [23310] = true,
    [7893] = true,
    [27725] = true,
    [8005] = true,
    [16072] = true,
    [12073] = true,
    [4067] = true,
    [20431] = true,
    [1026] = true,
    [16528] = true,
    [16656] = true,
    [8362] = true,
    [16784] = true,
    [12521] = true,
    [1062] = true,
    [8618] = true,
    [12745] = true,
    [8682] = true,
    [1090] = true,
    [1094] = true,
    [1098] = true,
    [1106] = true,
    [30093] = true,
    [8938] = true,
    [1122] = true,
    [9002] = true,
    [13225] = true,
    [22415] = true,
    [18448] = true,
    [18576] = true,
    [22799] = true,
    [22991] = true,
    [9482] = true,
    [23247] = true,
    [4950] = true,
    [24399] = true,
    [5110] = true,
    [5174] = true,
    [16657] = true,
    [20816] = true,
    [29134] = true,
    [5270] = true,
    [17169] = true,
    [10698] = true,
    [25807] = true,
    [17745] = true,
    [15049] = true,
    [26063] = true,
    [11082] = true,
    [15241] = true,
    [15305] = true,
    [22480] = true,
    [18449] = true,
    [11338] = true,
    [15497] = true,
    [5782] = true,
    [1450] = true,
    [23248] = true,
    [23312] = true,
    [15785] = true,
    [23632] = true,
    [12074] = true,
    [24208] = true,
    [20433] = true,
    [16402] = true,
    [16594] = true,
    [16658] = true,
    [8363] = true,
    [8395] = true,
    [12522] = true,
    [6278] = true,
    [16978] = true,
    [6310] = true,
    [6358] = true,
    [6422] = true,
    [17618] = true,
    [6470] = true,
    [6518] = true,
    [9003] = true,
    [13226] = true,
    [6630] = true,
    [13322] = true,
    [13482] = true,
    [6758] = true,
    [9483] = true,
    [13642] = true,
    [23249] = true,
    [23313] = true,
    [9739] = true,
    [9771] = true,
    [6950] = true,
    [19666] = true,
    [6982] = true,
    [9931] = true,
    [9995] = true,
    [14122] = true,
    [10059] = true,
    [20434] = true,
    [16531] = true,
    [10347] = true,
    [16659] = true,
    [24913] = true,
    [25297] = true,
    [1842] = true,
    [17235] = true,
    [10699] = true,
    [10795] = true,
    [10955] = true,
    [15114] = true,
    [15242] = true,
    [7638] = true,
    [18451] = true,
    [11339] = true,
    [15498] = true,
    [15530] = true,
    [7782] = true,
    [23250] = true,
    [11659] = true,
    [23442] = true,
    [19667] = true,
    [2006] = true,
    [2010] = true,
    [12075] = true,
    [8102] = true,
    [24274] = true,
    [20435] = true,
    [2052] = true,
    [2060] = true,
    [20627] = true,
    [16596] = true,
    [16660] = true,
    [8364] = true,
    [12491] = true,
    [12523] = true,
    [16980] = true,
    [25298] = true,
    [21331] = true,
    [12747] = true,
    [8780] = true,
    [8812] = true,
    [8940] = true,
    [9004] = true,
    [13227] = true,
    [13323] = true,
    [13419] = true,
    [22867] = true,
    [9484] = true,
    [23187] = true,
    [2396] = true,
    [9612] = true,
    [13899] = true,
    [13931] = true,
    [19860] = true,
    [9964] = true,
    [28242] = true,
    [28306] = true,
    [24275] = true,
    [2540] = true,
    [2548] = true,
    [14379] = true,
    [5159] = true,
    [10348] = true,
    [16661] = true,
    [16725] = true,
    [2660] = true,
    [2668] = true,
    [10700] = true,
    [10796] = true,
    [2740] = true,
    [2812] = true,
    [18453] = true,
    [11340] = true,
    [2860] = true,
    [15659] = true,
    [2908] = true,
    [11660] = true,
    [23508] = true,
    [2948] = true,
    [15915] = true,
    [19669] = true,
    [16075] = true,
    [12044] = true,
    [12076] = true,
    [3068] = true,
    [3084] = true,
    [3092] = true,
    [6199] = true,
    [6215] = true,
    [8365] = true,
    [12492] = true,
    [3132] = true,
    [3140] = true,
    [29331] = true,
    [12684] = true,
    [21397] = true,
    [8717] = true,
    [25748] = true,
    [6471] = true,
    [6487] = true,
    [8941] = true,
    [6535] = true,
    [3276] = true,
    [3292] = true,
    [13228] = true,
    [6631] = true,
    [3324] = true,
    [3332] = true,
    [22677] = true,
    [3356] = true,
    [3364] = true,
    [3372] = true,
    [13548] = true,
    [9485] = true,
    [13612] = true,
    [13644] = true,
    [3420] = true,
    [9613] = true,
    [3436] = true,
    [23509] = true,
    [13836] = true,
    [13868] = true,
    [6951] = true,
    [27860] = true,
    [3492] = true,
    [9901] = true,
    [9933] = true,
    [9997] = true,
    [28244] = true,
    [7079] = true,
    [3564] = true,
    [14380] = true,
    [20630] = true,
    [7223] = true,
    [16663] = true,
    [20822] = true,
    [3636] = true,
    [3644] = true,
    [3652] = true,
    [16983] = true,
    [29332] = true,
    [10605] = true,
    [7383] = true,
    [10701] = true,
    [10733] = true,
    [17559] = true,
    [3748] = true,
    [26069] = true,
    [11021] = true,
    [11085] = true,
    [18199] = true,
    [7639] = true,
    [7655] = true,
    [3844] = true,
    [3852] = true,
    [3860] = true,
    [3868] = true,
    [7751] = true,
    [22870] = true,
    [7799] = true,
    [7863] = true,
    [11661] = true,
    [11725] = true,
    [23510] = true,
    [23638] = true,
    [15980] = true,
    [19863] = true,
    [12045] = true,
    [8087] = true,
    [8103] = true,
    [4068] = true,
    [16] = true,
    [16408] = true,
    [8238] = true,
    [16600] = true,
    [16664] = true,
    [16728] = true,
    [8398] = true,
    [12525] = true,
    [16984] = true,
    [21143] = true,
    [12653] = true,
    [8686] = true,
    [17496] = true,
    [17560] = true,
    [8814] = true,
    [26006] = true,
    [26070] = true,
    [26134] = true,
    [4520] = true,
    [13229] = true,
    [18392] = true,
    [18456] = true,
    [18584] = true,
    [13421] = true,
    [13485] = true,
    [22999] = true,
    [23063] = true,
    [9614] = true,
    [23639] = true,
    [13901] = true,
    [13933] = true,
    [27990] = true,
    [19864] = true,
    [9966] = true,
    [20312] = true,
    [16409] = true,
    [20568] = true,
    [10318] = true,
    [5176] = true,
    [16665] = true,
    [16729] = true,
    [16985] = true,
    [21144] = true,
    [10702] = true,
    [10798] = true,
    [17561] = true,
    [21848] = true,
    [26071] = true,
    [15117] = true,
    [22168] = true,
    [15245] = true,
    [18457] = true,
    [11342] = true,
    [22744] = true,
    [15533] = true,
    [23000] = true,
    [11534] = true,
    [5784] = true,
    [27287] = true,
    [11726] = true,
    [15853] = true,
    [11790] = true,
    [19865] = true,
    [24024] = true,
    [12046] = true,
    [12078] = true,
    [28311] = true,
    [20313] = true,
    [20377] = true,
    [16410] = true,
    [16730] = true,
    [8399] = true,
    [12526] = true,
    [25112] = true,
    [16986] = true,
    [29335] = true,
    [25304] = true,
    [8687] = true,
    [17434] = true,
    [25688] = true,
    [17562] = true,
    [21913] = true,
    [26072] = true,
    [18138] = true,
    [13230] = true,
    [13262] = true,
    [6648] = true,
    [18458] = true,
    [6728] = true,
    [22937] = true,
    [13582] = true,
    [13646] = true,
    [27608] = true,
    [9743] = true,
    [27992] = true,
    [9935] = true,
    [20314] = true,
    [20378] = true,
    [7224] = true,
    [16667] = true,
    [16731] = true,
    [16795] = true,
    [7288] = true,
    [16987] = true,
    [10703] = true,
    [17435] = true,
    [10799] = true,
    [17563] = true,
    [17883] = true,
    [15118] = true,
    [11343] = true,
    [7752] = true,
    [15534] = true,
    [22938] = true,
    [7800] = true,
    [15662] = true,
    [7896] = true,
    [7928] = true,
    [7960] = true,
    [15982] = true,
    [8008] = true,
    [19867] = true,
    [12047] = true,
    [12079] = true,
    [8104] = true,
    [20315] = true,
    [20379] = true,
    [2053] = true,
    [2061] = true,
    [8272] = true,
    [4153] = true,
    [20763] = true,
    [16732] = true,
    [8400] = true,
    [16988] = true,
    [12655] = true,
    [17180] = true,
    [21403] = true,
    [8688] = true,
    [17564] = true,
    [21787] = true,
    [8880] = true,
    [17820] = true,
    [9200] = true,
    [9232] = true,
    [13519] = true,
    [9456] = true,
    [13583] = true,
    [2389] = true,
    [2397] = true,
    [9616] = true,
    [19484] = true,
    [13935] = true,
    [19868] = true,
    [9968] = true,
    [24091] = true,
    [10096] = true,
    [2541] = true,
    [2549] = true,
    [16413] = true,
    [5161] = true,
    [5177] = true,
    [20764] = true,
    [5273] = true,
    [2645] = true,
    [25307] = true,
    [2661] = true,
    [10704] = true,
    [10768] = true,
    [5401] = true,
    [17565] = true,
    [2741] = true,
    [11024] = true,
    [18205] = true,
    [2837] = true,
    [15535] = true,
    [11568] = true,
    [27291] = true,
    [2941] = true,
    [3013] = true,
    [6041] = true,
    [20317] = true,
    [20381] = true,
    [16367] = true,
    [8209] = true,
    [3085] = true,
    [3093] = true,
    [6201] = true,
    [3109] = true,
    [3117] = true,
    [16798] = true,
    [12528] = true,
    [8465] = true,
    [16990] = true,
    [12624] = true,
    [8593] = true,
    [3205] = true,
    [17566] = true,
    [3229] = true,
    [3237] = true,
    [6521] = true,
    [17950] = true,
    [3277] = true,
    [3293] = true,
    [3325] = true,
    [3333] = true,
    [22749] = true,
    [22813] = true,
    [3373] = true,
    [6777] = true,
    [9489] = true,
    [13648] = true,
    [3421] = true,
    [3429] = true,
    [6905] = true,
    [3477] = true,
    [3493] = true,
    [3501] = true,
    [9937] = true,
    [10001] = true,
    [7081] = true,
    [20318] = true,
    [20382] = true,
    [16415] = true,
    [3605] = true,
    [20702] = true,
    [20830] = true,
    [16799] = true,
    [7289] = true,
    [16991] = true,
    [25309] = true,
    [10673] = true,
    [10705] = true,
    [17503] = true,
    [17567] = true,
    [7481] = true,
    [10961] = true,
    [17951] = true,
    [7641] = true,
    [22430] = true,
    [18399] = true,
    [3845] = true,
    [3861] = true,
    [15472] = true,
    [7753] = true,
    [15536] = true,
    [7801] = true,
    [7817] = true,
    [15664] = true,
    [15728] = true,
    [11665] = true,
    [15792] = true,
    [11729] = true,
    [15856] = true,
    [11921] = true,
    [11985] = true,
    [12049] = true,
    [12081] = true,
    [8105] = true,
    [4061] = true,
    [4069] = true,
    [20319] = true,
    [16336] = true,
    [24670] = true,
    [20831] = true,
    [8402] = true,
    [66] = true,
    [16992] = true,
    [8690] = true,
    [8786] = true,
    [17952] = true,
    [4506] = true,
    [18656] = true,
    [1179] = true,
    [13617] = true,
    [13745] = true,
    [13841] = true,
    [13905] = true,
    [13937] = true,
    [4954] = true,
    [14033] = true,
    [9970] = true,
    [20320] = true,
    [20384] = true,
    [10258] = true,
    [10322] = true,
    [5178] = true,
    [25055] = true,
    [16993] = true,
    [25311] = true,
    [10674] = true,
    [10706] = true,
    [17505] = true,
    [30046] = true,
    [15057] = true,
    [30174] = true,
    [17953] = true,
    [18081] = true,
    [1403] = true,
    [18401] = true,
    [18657] = true,
    [11410] = true,
    [15537] = true,
    [11538] = true,
    [15665] = true,
    [1451] = true,
    [23392] = true,
    [11730] = true,
    [11762] = true,
    [27871] = true,
    [11922] = true,
    [16081] = true,
    [12050] = true,
    [12082] = true,
    [20321] = true,
    [20385] = true,
    [16418] = true,
    [8275] = true,
    [6202] = true,
    [12466] = true,
    [20897] = true,
    [16866] = true,
    [12562] = true,
    [16994] = true,
    [8595] = true,
    [8691] = true,
    [17506] = true,
    [6490] = true,
    [18082] = true,
    [18402] = true,
    [26656] = true,
    [18658] = true,
    [13522] = true,
    [6778] = true,
    [13746] = true,
    [9811] = true,
    [9875] = true,
    [14034] = true,
    [10003] = true,
    [7098] = true,
    [28448] = true,
    [20322] = true,
    [20386] = true,
    [7162] = true,
    [16419] = true,
    [113] = true,
    [20770] = true,
    [10451] = true,
    [16995] = true,
    [17187] = true,
    [10675] = true,
    [10707] = true,
    [116] = true,
    [7434] = true,
    [25953] = true,
    [10963] = true,
    [118] = true,
    [18211] = true,
    [22434] = true,
    [18403] = true,
    [7754] = true,
    [11443] = true,
    [7786] = true,
    [7802] = true,
    [7818] = true,
    [15730] = true,
    [11667] = true,
    [15794] = true,
    [11763] = true,
    [7962] = true,
    [7978] = true,
    [7994] = true,
    [11923] = true,
    [16082] = true,
    [126] = true,
    [12083] = true,
    [8106] = true,
    [8138] = true,
    [20323] = true,
    [12243] = true,
    [2054] = true,
    [20707] = true,
    [16868] = true,
    [21027] = true,
    [133] = true,
    [8532] = true,
    [134] = true,
    [8596] = true,
    [12883] = true,
    [8820] = true,
    [25954] = true,
    [8980] = true,
    [4539] = true,
    [143] = true,
    [18404] = true,
    [145] = true,
    [18596] = true,
    [22883] = true,
    [22947] = true,
    [2366] = true,
    [2406] = true,
    [13939] = true,
    [9876] = true,
    [4971] = true,
    [9972] = true,
    [2542] = true,
    [20388] = true,
    [10228] = true,
    [10324] = true,
    [5179] = true,
    [5195] = true,
    [14515] = true,
    [10452] = true,
    [16869] = true,
    [5275] = true,
    [2662] = true,
    [10676] = true,
    [10708] = true,
    [5403] = true,
    [21668] = true,
    [10964] = true,
    [172] = true,
    [18149] = true,
    [26403] = true,
    [5627] = true,
    [2838] = true,
    [11444] = true,
    [2878] = true,
    [11604] = true,
    [11668] = true,
    [184] = true,
    [23652] = true,
    [11892] = true,
    [19877] = true,
    [19941] = true,
    [12052] = true,
    [12084] = true,
    [20389] = true,
    [3070] = true,
    [24612] = true,
    [8245] = true,
    [8277] = true,
    [3110] = true,
    [12468] = true,
    [20901] = true,
    [25124] = true,
    [25316] = true,
    [8693] = true,
    [3206] = true,
    [17510] = true,
    [8789] = true,
    [8949] = true,
    [205] = true,
    [18086] = true,
    [3294] = true,
    [18214] = true,
    [22373] = true,
    [6635] = true,
    [3326] = true,
    [3334] = true,
    [22757] = true,
    [13524] = true,
    [9461] = true,
    [3398] = true,
    [13620] = true,
    [13748] = true,
    [23461] = true,
    [6907] = true,
    [23653] = true,
    [9813] = true,
    [3494] = true,
    [3502] = true,
    [19942] = true,
    [10005] = true,
    [20326] = true,
    [14292] = true,
    [3606] = true,
    [20902] = true,
    [25125] = true,
    [7355] = true,
    [10677] = true,
    [10709] = true,
    [3718] = true,
    [14900] = true,
    [10837] = true,
    [10869] = true,
    [3750] = true,
    [10933] = true,
    [10965] = true,
    [26085] = true,
    [15124] = true,
    [7643] = true,
    [18407] = true,
    [22566] = true,
    [3854] = true,
    [3862] = true,
    [7739] = true,
    [7755] = true,
    [7771] = true,
    [15668] = true,
    [11605] = true,
    [7867] = true,
    [7931] = true,
    [11829] = true,
    [16084] = true,
    [19943] = true,
    [12053] = true,
    [12085] = true,
    [4054] = true,
    [4062] = true,
    [8139] = true,
    [20327] = true,
    [16340] = true,
    [4094] = true,
    [8246] = true,
    [16552] = true,
    [8342] = true,
    [16744] = true,
    [8406] = true,
    [25126] = true,
    [21159] = true,
    [8598] = true,
    [271] = true,
    [8694] = true,
    [8758] = true,
    [25958] = true,
    [8950] = true,
    [18088] = true,
    [13237] = true,
    [18408] = true,
    [22567] = true,
    [22695] = true,
    [22759] = true,
    [13653] = true,
    [9654] = true,
    [9750] = true,
    [9814] = true,
    [13941] = true,
    [19816] = true,
    [9942] = true,
    [9974] = true,
    [20008] = true,
    [24359] = true,
    [10166] = true,
    [14293] = true,
    [10326] = true,
    [5180] = true,
    [5196] = true,
    [16745] = true,
    [20904] = true,
    [25127] = true,
    [5276] = true,
    [331] = true,
    [332] = true,
    [10678] = true,
    [10710] = true,
    [21544] = true,
    [339] = true,
    [21736] = true,
    [10934] = true,
    [5484] = true,
    [26087] = true,
    [15125] = true,
    [18089] = true,
    [348] = true,
    [18217] = true,
    [18409] = true,
    [5676] = true,
    [15797] = true,
    [5884] = true,
    [23592] = true,
    [16053] = true,
    [20009] = true,
    [12086] = true,
    [6060] = true,
    [20201] = true,
    [20329] = true,
    [20393] = true,
    [24680] = true,
    [16554] = true,
    [20777] = true,
    [16746] = true,
    [6252] = true,
    [25128] = true,
    [21097] = true,
    [21161] = true,
    [17194] = true,
    [6412] = true,
    [25704] = true,
    [403] = true,
    [25960] = true,
    [8951] = true,
    [22313] = true,
    [9143] = true,
    [18410] = true,
    [22761] = true,
    [421] = true,
    [6780] = true,
    [13622] = true,
    [9591] = true,
    [19434] = true,
    [13846] = true,
    [13878] = true,
    [9879] = true,
    [10007] = true,
    [7068] = true,
    [7084] = true,
    [7132] = true,
    [20394] = true,
    [14326] = true,
    [20714] = true,
    [10391] = true,
    [455] = true,
    [29160] = true,
    [457] = true,
    [458] = true,
    [459] = true,
    [17195] = true,
    [10679] = true,
    [10711] = true,
    [7420] = true,
    [7484] = true,
    [17707] = true,
    [470] = true,
    [471] = true,
    [472] = true,
    [474] = true,
    [15254] = true,
    [18411] = true,
    [11479] = true,
    [27241] = true,
    [491] = true,
    [15734] = true,
    [11671] = true,
    [19435] = true,
    [7948] = true,
    [11895] = true,
    [19755] = true,
    [12055] = true,
    [12087] = true,
    [507] = true,
    [12151] = true,
    [509] = true,
    [20395] = true,
    [12279] = true,
    [2055] = true,
    [4141] = true,
    [12471] = true,
    [8408] = true,
    [4221] = true,
    [25130] = true,
    [21099] = true,
    [536] = true,
    [8600] = true,
    [25514] = true,
    [8760] = true,
    [17708] = true,
    [556] = true,
    [21931] = true,
    [26218] = true,
    [18412] = true,
    [578] = true,
    [18540] = true,
    [13399] = true,
    [13463] = true,
    [18796] = true,
    [13527] = true,
    [23083] = true,
    [596] = true,
    [598] = true,
    [13687] = true,
    [23339] = true,
    [606] = true,
    [13815] = true,
    [13943] = true,
    [9912] = true,
    [14103] = true,
    [2543] = true,
    [20396] = true,
    [14327] = true,
    [16429] = true,
    [2575] = true,
    [10328] = true,
    [20716] = true,
    [10392] = true,
    [5213] = true,
    [2663] = true,
    [10680] = true,
    [10712] = true,
    [17453] = true,
    [10840] = true,
    [17709] = true,
    [25963] = true,
    [686] = true,
    [15095] = true,
    [690] = true,
    [692] = true,
    [698] = true,
    [700] = true,
    [18413] = true,
    [2823] = true,
    [18541] = true,
    [710] = true,
    [712] = true,
    [11416] = true,
    [11480] = true,
    [18989] = true,
    [724] = true,
    [11672] = true,
    [15799] = true,
    [15863] = true,
    [2951] = true,
    [19821] = true,
    [3007] = true,
    [12056] = true,
    [12088] = true,
    [16247] = true,
    [12248] = true,
    [12280] = true,
    [3079] = true,
    [3087] = true,
    [3095] = true,
    [20717] = true,
    [25004] = true,
    [29163] = true,
    [3143] = true,
    [3207] = true,
    [8793] = true,
    [6461] = true,
    [25964] = true,
    [3263] = true,
    [818] = true,
    [9081] = true,
    [6637] = true,
    [6653] = true,
    [3335] = true,
    [6717] = true,
    [13464] = true,
    [13528] = true,
    [18990] = true,
    [9657] = true,
    [6909] = true,
    [6925] = true,
    [19566] = true,
    [6957] = true,
    [3495] = true,
    [14008] = true,
    [3511] = true,
    [10009] = true,
    [20078] = true,
    [7101] = true,
    [20270] = true,
    [3567] = true,
    [20398] = true,
    [3583] = true,
    [3607] = true,
    [7277] = true,
    [3655] = true,
    [10681] = true,
    [10713] = true,
    [7421] = true,
    [930] = true,
    [932] = true,
    [10841] = true,
    [10873] = true,
    [25965] = true,
    [10969] = true,
    [15128] = true,
    [18159] = true,
    [7629] = true,
    [7645] = true,
    [18351] = true,
    [18415] = true,
    [3847] = true,
    [3855] = true,
    [3863] = true,
    [968] = true,
    [11417] = true,
    [7821] = true,
    [7853] = true,
    [984] = true,
    [7901] = true,
    [990] = true,
    [7949] = true,
    [996] = true,
    [1002] = true,
    [16056] = true,
    [19887] = true,
    [20015] = true,
    [12089] = true,
    [4055] = true,
    [28461] = true,
    [20399] = true,
    [8282] = true,
    [24942] = true,
    [12505] = true,
    [12537] = true,
    [29229] = true,
    [1064] = true,
    [25262] = true,
    [4286] = true,
    [1084] = true,
    [1088] = true,
    [12825] = true,
    [8762] = true,
    [8858] = true,
    [1112] = true,
    [21935] = true,
    [1124] = true,
    [22127] = true,
    [4526] = true,
    [18416] = true,
    [13529] = true,
    [18992] = true,
    [13657] = true,
    [13689] = true,
    [9658] = true,
    [13817] = true,
    [23663] = true,
    [9818] = true,
    [13945] = true,
    [4974] = true,
    [20016] = true,
    [28270] = true,
    [24239] = true,
    [10138] = true,
    [28526] = true,
    [20400] = true,
    [16497] = true,
    [16561] = true,
    [20720] = true,
    [20848] = true,
    [5262] = true,
    [17009] = true,
    [25263] = true,
    [17137] = true,
    [10682] = true,
    [10714] = true,
    [17393] = true,
    [10842] = true,
    [10874] = true,
    [25967] = true,
    [18417] = true,
    [18609] = true,
    [11418] = true,
    [23088] = true,
    [1452] = true,
    [11642] = true,
    [15801] = true,
    [27567] = true,
    [15865] = true,
    [11802] = true,
    [23664] = true,
    [11962] = true,
    [19889] = true,
    [20017] = true,
    [12090] = true,
    [16249] = true,
    [28463] = true,
    [20401] = true,
    [1536] = true,
    [1540] = true,
    [16498] = true,
    [8283] = true,
    [6222] = true,
    [20849] = true,
    [6254] = true,
    [6270] = true,
    [17138] = true,
    [6350] = true,
    [6366] = true,
    [21425] = true,
    [6414] = true,
    [12890] = true,
    [25968] = true,
    [6510] = true,
    [8955] = true,
    [6638] = true,
    [18418] = true,
    [22705] = true,
    [13530] = true,
    [23089] = true,
    [6814] = true,
    [6894] = true,
    [27632] = true,
    [13882] = true,
    [19890] = true,
    [9979] = true,
    [10011] = true,
    [28272] = true,
    [10139] = true,
    [20402] = true,
    [1804] = true,
    [10395] = true,
    [25073] = true,
    [17203] = true,
    [10683] = true,
    [10715] = true,
    [17459] = true,
    [7454] = true,
    [10875] = true,
    [25969] = true,
    [17843] = true,
    [26097] = true,
    [11067] = true,
    [11131] = true,
    [7630] = true,
    [1916] = true,
    [18419] = true,
    [11355] = true,
    [11419] = true,
    [27057] = true,
    [23090] = true,
    [11643] = true,
    [15802] = true,
    [7918] = true,
    [7934] = true,
    [23666] = true,
    [11899] = true,
    [16058] = true,
    [12059] = true,
    [12091] = true,
    [24242] = true,
    [16250] = true,
    [8142] = true,
    [20403] = true,
    [28785] = true,
    [16564] = true,
    [8348] = true,
    [8444] = true,
    [2120] = true,
    [21171] = true,
    [12667] = true,
    [8604] = true,
    [17460] = true,
    [8764] = true,
    [30001] = true,
    [25970] = true,
    [26098] = true,
    [8988] = true,
    [18420] = true,
    [22643] = true,
    [27058] = true,
    [2368] = true,
    [23091] = true,
    [9532] = true,
    [13659] = true,
    [13819] = true,
    [23667] = true,
    [13915] = true,
    [9852] = true,
    [9916] = true,
    [9980] = true,
    [20020] = true,
    [10140] = true,
    [2544] = true,
    [20404] = true,
    [2576] = true,
    [16565] = true,
    [16629] = true,
    [10396] = true,
    [24947] = true,
    [20916] = true,
    [17141] = true,
    [2664] = true,
    [10684] = true,
    [10716] = true,
    [17461] = true,
    [10844] = true,
    [10876] = true,
    [25971] = true,
    [15067] = true,
    [5567] = true,
    [26419] = true,
    [18421] = true,
    [2824] = true,
    [11356] = true,
    [11420] = true,
    [27059] = true,
    [15611] = true,
    [11548] = true,
    [23220] = true,
    [2912] = true,
    [11836] = true,
    [19701] = true,
    [23860] = true,
    [16059] = true,
    [12060] = true,
    [12092] = true,
    [6063] = true,
    [6127] = true,
    [3072] = true,
    [3080] = true,
    [3088] = true,
    [3096] = true,
    [6223] = true,
    [3120] = true,
    [6255] = true,
    [8445] = true,
    [17142] = true,
    [6351] = true,
    [6415] = true,
    [8797] = true,
    [6463] = true,
    [25972] = true,
    [3256] = true,
    [3264] = true,
    [9053] = true,
    [3296] = true,
    [3320] = true,
    [3328] = true,
    [6671] = true,
    [22709] = true,
    [3368] = true,
    [3376] = true,
    [3400] = true,
    [3408] = true,
    [23221] = true,
    [13692] = true,
    [27572] = true,
    [3472] = true,
    [9853] = true,
    [3496] = true,
    [3504] = true,
    [28148] = true,
    [10013] = true,
    [3552] = true,
    [10205] = true,
    [3584] = true,
    [20790] = true,
    [7279] = true,
    [7295] = true,
    [7359] = true,
    [3696] = true,
    [10717] = true,
    [7439] = true,
    [3728] = true,
    [7487] = true,
    [25973] = true,
    [15228] = true,
    [15292] = true,
    [3840] = true,
    [3848] = true,
    [3856] = true,
    [3864] = true,
    [3872] = true,
    [22902] = true,
    [7791] = true,
    [15612] = true,
    [23094] = true,
    [7855] = true,
    [7919] = true,
    [7935] = true,
    [7951] = true,
    [7967] = true,
    [27829] = true,
    [23862] = true,
    [16060] = true,
    [24054] = true,
    [20023] = true,
    [12093] = true,
    [4064] = true,
    [24374] = true,
    [20407] = true,
    [20535] = true,
    [8286] = true,
    [20791] = true,
    [12509] = true,
    [8446] = true,
    [21047] = true,
    [21175] = true,
    [8606] = true,
    [4320] = true,
    [17464] = true,
    [8766] = true,
    [25974] = true,
    [26102] = true,
    [30732] = true,
    [30156] = true,
    [30096] = true,
    [30091] = true,
    [18424] = true,
    [9472] = true,
    [30047] = true,
    [16742] = true,
    [29334] = true,
    [29333] = true,
    [13501] = true,
    [5272] = true,
    [28995] = true,
    [18153] = true,
    [28739] = true,
    [13661] = true,
    [13693] = true,
    [7489] = true,
    [8467] = true,
    [28505] = true,
    [28487] = true,
    [9758] = true,
    [28482] = true,
    [13917] = true,
    [23799] = true,
    [6297] = true,
    [4960] = true,
    [9950] = true,
    [3693] = true,
    [14109] = true,
    [20698] = true,
    [28462] = true,
    [461] = true,
    [28328] = true,
    [28305] = true,
    [10206] = true,
    [28304] = true,
    [20536] = true,
    [6651] = true,
    [28286] = true,
    [16633] = true,
    [20792] = true,
    [28271] = true,
    [28243] = true,
    [25079] = true,
    [5264] = true,
    [5280] = true,
    [28210] = true,
    [10622] = true,
    [28209] = true,
    [10686] = true,
    [10718] = true,
    [28208] = true,
    [17465] = true,
    [28207] = true,
    [28205] = true,
    [28146] = true,
    [28133] = true,
    [15037] = true,
    [28089] = true,
    [5504] = true,
    [1096] = true,
    [27891] = true,
    [18105] = true,
    [27890] = true,
    [15261] = true,
    [15293] = true,
    [27870] = true,
    [21160] = true,
    [22945] = true,
    [3243] = true,
    [11358] = true,
    [27830] = true,
    [27794] = true,
    [15549] = true,
    [27760] = true,
    [27724] = true,
    [27723] = true,
    [27721] = true,
    [23224] = true,
    [9435] = true,
    [27660] = true,
    [3503] = true,
    [27637] = true,
    [15869] = true,
    [27624] = true,
    [15933] = true,
    [27831] = true,
    [23800] = true,
    [19769] = true,
    [28023] = true,
    [27588] = true,
    [18440] = true,
    [12062] = true,
    [5186] = true,
    [6064] = true,
    [4065] = true,
    [27527] = true,
    [27106] = true,
    [27100] = true,
    [16381] = true,
    [27098] = true,
    [24696] = true,
    [16570] = true,
    [26616] = true,
    [20793] = true,
    [19851] = true,
    [4983] = true,
    [12542] = true,
    [16168] = true,
    [26437] = true,
    [26436] = true,
    [26435] = true,
    [8607] = true,
    [17274] = true,
    [12766] = true,
    [26434] = true,
    [6416] = true,
    [26433] = true,
    [8799] = true,
    [26432] = true,
    [26431] = true,
    [25976] = true,
    [21945] = true,
    [26430] = true,
    [26429] = true,
    [26407] = true,
    [9055] = true,
    [12066] = true,
    [23246] = true,
    [26298] = true,
    [24093] = true,
    [6656] = true,
    [26192] = true,
    [26103] = true,
    [26086] = true,
    [13438] = true,
    [3108] = true,
    [26055] = true,
    [16724] = true,
    [20035] = true,
    [26010] = true,
    [23161] = true,
    [23225] = true,
    [25985] = true,
    [691] = true,
    [25983] = true,
    [6896] = true,
    [13822] = true,
    [20819] = true,
    [25981] = true,
    [27832] = true,
    [23801] = true,
    [25979] = true,
    [24139] = true,
    [25183] = true,
    [9983] = true,
    [10015] = true,
    [25966] = true,
    [25962] = true,
    [25961] = true,
    [7120] = true,
    [25959] = true,
    [10207] = true,
    [25849] = true,
    [25841] = true,
    [25839] = true,
    [25808] = true,
    [25804] = true,
    [20794] = true,
    [25722] = true,
    [25664] = true,
    [25081] = true,
    [20821] = true,
    [7328] = true,
    [23316] = true,
    [10623] = true,
    [25424] = true,
    [10687] = true,
    [10719] = true,
    [16006] = true,
    [25347] = true,
    [25314] = true,
    [25306] = true,
    [7488] = true,
    [10911] = true,
    [25977] = true,
    [25181] = true,
    [25180] = true,
    [25178] = true,
    [25177] = true,
    [25162] = true,
    [15230] = true,
    [15262] = true,
    [15294] = true,
    [18363] = true,
    [25146] = true,
    [25129] = true,
    [20829] = true,
    [7728] = true,
    [22336] = true,
    [23085] = true,
    [7776] = true,
    [23391] = true,
    [25082] = true,
    [23098] = true,
    [25080] = true,
    [25078] = true,
    [25074] = true,
    [25072] = true,
    [25054] = true,
    [25052] = true,
    [27641] = true,
    [7952] = true,
    [11839] = true,
    [25030] = true,
    [8000] = true,
    [8016] = true,
    [25018] = true,
    [16094] = true,
    [24997] = true,
    [12063] = true,
    [24995] = true,
    [16969] = true,
    [24314] = true,
    [24914] = true,
    [24912] = true,
    [24903] = true,
    [24902] = true,
    [24901] = true,
    [8256] = true,
    [24801] = true,
    [24706] = true,
    [8352] = true,
    [24668] = true,
    [4209] = true,
    [12543] = true,
    [2121] = true,
    [16652] = true,
    [24418] = true,
    [24358] = true,
    [24334] = true,
    [23102] = true,
    [24209] = true,
    [24189] = true,
    [8736] = true,
    [8768] = true,
    [8800] = true,
    [24173] = true,
    [24140] = true,
    [25978] = true,
    [24138] = true,
    [10329] = true,
    [24136] = true,
    [26234] = true,
    [18108] = true,
    [24092] = true,
    [24011] = true,
    [7395] = true,
    [23954] = true,
    [5274] = true,
    [23804] = true,
    [13375] = true,
    [23803] = true,
    [13439] = true,
    [23802] = true,
    [13503] = true,
    [23787] = true,
    [2369] = true,
    [23665] = true,
    [13631] = true,
    [2393] = true,
    [13695] = true,
    [23662] = true,
    [23650] = true,
    [23637] = true,
    [19452] = true,
    [23636] = true,
    [13887] = true,
    [23633] = true,
    [9856] = true,
    [9888] = true,
    [9920] = true,
    [9952] = true,
    [23629] = true,
    [20028] = true,
    [23531] = true,
    [23530] = true,
    [6413] = true,
    [10144] = true,
    [2545] = true,
    [3336] = true,
    [10771] = true,
    [5137] = true,
    [2577] = true,
    [5169] = true,
    [5185] = true,
    [2601] = true,
    [3570] = true,
    [8153] = true,
    [25083] = true,
    [5265] = true,
    [2641] = true,
    [23381] = true,
    [23338] = true,
    [2665] = true,
    [2673] = true,
    [10720] = true,
    [17405] = true,
    [25659] = true,
    [23315] = true,
    [23314] = true,
    [10880] = true,
    [10912] = true,
    [2737] = true,
    [23309] = true,
    [5505] = true,
    [23252] = true,
    [23251] = true,
    [26299] = true,
    [23243] = true,
    [15263] = true,
    [15295] = true,
    [23241] = true,
    [15119] = true,
    [23239] = true,
    [23238] = true,
    [2841] = true,
    [15451] = true,
    [23228] = true,
    [23227] = true,
    [23223] = true,
    [5761] = true,
    [15647] = true,
    [19069] = true,
    [5809] = true,
    [15743] = true,
    [23222] = true,
    [23219] = true,
    [23214] = true,
    [23206] = true,
    [23189] = true,
    [15935] = true,
    [19645] = true,
    [15999] = true,
    [16031] = true,
    [23130] = true,
    [19862] = true,
    [20773] = true,
    [20029] = true,
    [12096] = true,
    [24252] = true,
    [23093] = true,
    [23092] = true,
    [23087] = true,
    [6129] = true,
    [3073] = true,
    [3081] = true,
    [23086] = true,
    [3097] = true,
    [3105] = true,
    [20797] = true,
    [3121] = true,
    [6257] = true,
    [25084] = true,
    [23084] = true,
    [6305] = true,
    [17086] = true,
    [23030] = true,
    [6353] = true,
    [23013] = true,
    [23012] = true,
    [22990] = true,
    [6417] = true,
    [22989] = true,
    [22949] = true,
    [3233] = true,
    [22946] = true,
    [25980] = true,
    [22909] = true,
    [22876] = true,
    [22869] = true,
    [22868] = true,
    [22866] = true,
    [3297] = true,
    [18238] = true,
    [22796] = true,
    [3321] = true,
    [22790] = true,
    [3337] = true,
    [18166] = true,
    [22717] = true,
    [13440] = true,
    [22750] = true,
    [6753] = true,
    [13536] = true,
    [9473] = true,
    [22724] = true,
    [9744] = true,
    [23229] = true,
    [3513] = true,
    [20405] = true,
    [7448] = true,
    [6897] = true,
    [22718] = true,
    [22710] = true,
    [22686] = true,
    [22678] = true,
    [13952] = true,
    [9889] = true,
    [3505] = true,
    [19902] = true,
    [9985] = true,
    [7057] = true,
    [3537] = true,
    [22665] = true,
    [22662] = true,
    [7121] = true,
    [22661] = true,
    [22651] = true,
    [22539] = true,
    [22478] = true,
    [22458] = true,
    [22425] = true,
    [16639] = true,
    [22421] = true,
    [22414] = true,
    [22357] = true,
    [25085] = true,
    [3657] = true,
    [22334] = true,
    [22275] = true,
    [22167] = true,
    [13900] = true,
    [3697] = true,
    [10721] = true,
    [22048] = true,
    [21971] = true,
    [7457] = true,
    [10849] = true,
    [10881] = true,
    [17727] = true,
    [15040] = true,
    [8791] = true,
    [17919] = true,
    [16080] = true,
    [3695] = true,
    [18111] = true,
    [15232] = true,
    [18239] = true,
    [15296] = true,
    [21807] = true,
    [18431] = true,
    [3849] = true,
    [3857] = true,
    [7729] = true,
    [7745] = true,
    [7761] = true,
    [21667] = true,
    [7793] = true,
    [21565] = true,
    [15648] = true,
    [21559] = true,
    [7857] = true,
    [21549] = true,
    [21402] = true,
    [21369] = true,
    [19391] = true,
    [21367] = true,
    [5514] = true,
    [21162] = true,
    [27837] = true,
    [21096] = true,
    [19775] = true,
    [21073] = true,
    [12001] = true,
    [21072] = true,
    [20031] = true,
    [21071] = true,
    [21068] = true,
    [8129] = true,
    [21066] = true,
    [19774] = true,
    [21048] = true,
    [20903] = true,
    [20543] = true,
    [4130] = true,
    [20900] = true,
    [16640] = true,
    [20890] = true,
    [10696] = true,
    [16083] = true,
    [25086] = true,
    [16960] = true,
    [20827] = true,
    [20826] = true,
    [20825] = true,
    [20824] = true,
    [12737] = true,
    [8674] = true,
    [20823] = true,
    [25662] = true,
    [8770] = true,
    [8802] = true,
    [20820] = true,
    [17728] = true,
    [25982] = true,
    [20817] = true,
    [17920] = true,
    [8994] = true,
    [20815] = true,
    [18112] = true,
    [20813] = true,
    [18240] = true,
    [15859] = true,
    [5188] = true,
    [17509] = true,
    [3210] = true,
    [18560] = true,
    [22719] = true,
    [6576] = true,
    [3694] = true,
    [20787] = true,
    [20776] = true,
    [9474] = true,
    [23103] = true,
    [20772] = true,
    [20765] = true,
    [20762] = true,
    [20757] = true,
    [20756] = true,
    [20755] = true,
    [20752] = true,
    [13857] = true,
    [20748] = true,
    [20747] = true,
    [9858] = true,
    [20742] = true,
    [4962] = true,
    [9954] = true,
    [9986] = true,
    [20032] = true,
    [5026] = true,
    [20739] = true,
    [20733] = true,
    [28478] = true,
    [20696] = true,
    [5106] = true,
    [20695] = true,
    [20692] = true,
    [20685] = true,
    [16965] = true,
    [16641] = true,
    [20800] = true,
    [20669] = true,
    [10466] = true,
    [20665] = true,
    [5266] = true,
    [20657] = true,
    [20656] = true,
    [20629] = true,
    [20626] = true,
    [20012] = true,
    [20609] = true,
    [20604] = true,
    [17473] = true,
    [20565] = true,
    [10850] = true,
    [2672] = true,
    [20432] = true,
    [10946] = true,
    [20430] = true,
    [17921] = true,
    [20429] = true,
    [1385] = true,
    [20428] = true,
    [20426] = true,
    [15265] = true,
    [11202] = true,
    [18617] = true,
    [20408] = true,
    [20406] = true,
    [5666] = true,
    [22720] = true,
    [20397] = true,
    [20392] = true,
    [20391] = true,
    [15585] = true,
    [20390] = true,
    [20387] = true,
    [20383] = true,
    [1453] = true,
    [3065] = true,
    [3069] = true,
    [5669] = true,
    [8366] = true,
    [3133] = true,
    [18439] = true,
    [6296] = true,
    [11986] = true,
    [14106] = true,
    [16033] = true,
    [17140] = true,
    [6352] = true,
    [15267] = true,
    [20033] = true,
    [1513] = true,
    [10685] = true,
    [20324] = true,
    [8782] = true,
    [19859] = true,
    [12258] = true,
    [24576] = true,
    [16450] = true,
    [8259] = true,
    [20316] = true,
    [16642] = true,
    [20801] = true,
    [6632] = true,
    [11341] = true,
    [6274] = true,
    [8483] = true,
    [6306] = true,
    [11357] = true,
    [16597] = true,
    [3361] = true,
    [13321] = true,
    [3377] = true,
    [20296] = true,
    [6418] = true,
    [17196] = true,
    [3397] = true,
    [13628] = true,
    [20051] = true,
    [25984] = true,
    [26048] = true,
    [17922] = true,
    [20014] = true,
    [15979] = true,
    [13948] = true,
    [3497] = true,
    [18242] = true,
    [6626] = true,
    [4961] = true,
    [18434] = true,
    [11981] = true,
    [6690] = true,
    [22721] = true,
    [13442] = true,
    [12061] = true,
    [18818] = true,
    [13538] = true,
    [12077] = true,
    [20030] = true,
    [20026] = true,
    [20025] = true,
    [13698] = true,
    [3565] = true,
    [20024] = true,
    [13794] = true,
    [19750] = true,
    [13858] = true,
    [13890] = true,
    [10254] = true,
    [16502] = true,
    [16662] = true,
    [3658] = true,
    [8367] = true,
    [9987] = true,
    [20034] = true,
    [14146] = true,
    [28352] = true,
    [7106] = true,
    [28480] = true,
    [10179] = true,
    [20006] = true,
    [7408] = true,
    [20013] = true,
    [20610] = true,
    [17462] = true,
    [16643] = true,
    [20802] = true,
    [10435] = true,
    [10467] = true,
    [3729] = true,
    [3733] = true,
    [468] = true,
    [20011] = true,
    [20010] = true,
    [3506] = true,
    [1849] = true,
    [19982] = true,
    [7426] = true,
    [10787] = true,
    [14914] = true,
    [17229] = true,
    [7648] = true,
    [10915] = true,
    [10947] = true,
    [21954] = true,
    [17923] = true,
    [8368] = true,
    [19970] = true,
    [3841] = true,
    [15234] = true,
    [15266] = true,
    [19968] = true,
    [3869] = true,
    [18435] = true,
    [3873] = true,
    [14807] = true,
    [22722] = true,
    [9487] = true,
    [19940] = true,
    [19939] = true,
    [15586] = true,
    [7992] = true,
    [23106] = true,
    [19874] = true,
    [8040] = true,
    [19873] = true,
    [8088] = true,
    [19866] = true,
    [27585] = true,
    [18327] = true,
    [15906] = true,
    [512] = true,
    [15970] = true,
    [18163] = true,
    [2005] = true,
    [19861] = true,
    [16098] = true,
    [19971] = true,
    [12067] = true,
    [15253] = true,
    [19848] = true,
    [14621] = true,
    [28481] = true,
    [17273] = true,
    [12259] = true,
    [2050] = true,
    [18438] = true,
    [4131] = true,
    [8292] = true,
    [12419] = true,
    [8356] = true,
    [6620] = true,
    [548] = true,
    [8784] = true,
    [21059] = true,
    [19772] = true,
    [8912] = true,
    [12675] = true,
    [19668] = true,
    [12739] = true,
    [579] = true,
    [580] = true,
    [15453] = true,
    [8772] = true,
    [8804] = true,
    [16601] = true,
    [10688] = true,
    [30081] = true,
    [9488] = true,
    [17924] = true,
    [2386] = true,
    [9552] = true,
    [18960] = true,
    [9092] = true,
    [13219] = true,
    [17450] = true,
    [605] = true,
    [9220] = true,
    [18500] = true,
    [18564] = true,
    [22723] = true,
    [12064] = true,
    [17454] = true,
    [18991] = true,
    [2362] = true,
    [2538] = true,
    [2546] = true,
    [13635] = true,
    [2394] = true,
    [18702] = true,
    [9636] = true,
    [2738] = true,
    [27586] = true,
    [18763] = true,
    [2742] = true,
    [18809] = true,
    [8401] = true,
    [23811] = true,
    [18647] = true,
    [18241] = true,
    [9956] = true,
    [19972] = true,
    [20036] = true,
    [2670] = true,
    [1056] = true,
    [18444] = true,
    [10148] = true,
    [10180] = true,
    [20420] = true,
    [20484] = true,
    [28738] = true,
    [2578] = true,
    [10340] = true,
    [16645] = true,
    [2602] = true,
    [10436] = true,
    [10468] = true,
    [688] = true,
    [5267] = true,
    [5506] = true,
    [21188] = true,
    [17157] = true,
    [2666] = true,
    [2674] = true,
    [695] = true,
    [7633] = true,
    [5395] = true,
    [701] = true,
    [17501] = true,
    [705] = true,
    [10916] = true,
    [15043] = true,
    [7769] = true,
    [18455] = true,
    [7865] = true,
    [18454] = true,
    [18452] = true,
    [18450] = true,
    [18245] = true,
    [7929] = true,
    [18447] = true,
    [18437] = true,
    [18446] = true,
    [9857] = true,
    [18629] = true,
    [5699] = true,
    [9921] = true,
    [3018] = true,
    [15587] = true,
    [5763] = true,
    [8089] = true,
    [7788] = true,
    [8137] = true,
    [10145] = true,
    [15779] = true,
    [23428] = true,
    [27587] = true,
    [3066] = true,
    [3074] = true,
    [16412] = true,
    [14847] = true,
    [3086] = true,
    [3094] = true,
    [19845] = true,
    [16099] = true,
    [19973] = true,
    [12068] = true,
    [18436] = true,
    [10945] = true,
    [18423] = true,
    [18422] = true,
    [8010] = true,
    [12260] = true,
    [16390] = true,
    [3082] = true,
    [3090] = true,
    [8293] = true,
    [16646] = true,
    [20805] = true,
    [29059] = true,
    [5208] = true,
    [18414] = true,
    [3146] = true,
    [6530] = true,
    [3278] = true,
    [17158] = true,
    [8613] = true,
    [12740] = true,
    [8677] = true,
    [18405] = true,
    [6419] = true,
    [6650] = true,
    [3330] = true,
    [877] = true,
    [14053] = true,
    [8901] = true,
    [13028] = true,
    [849] = true,
    [8090] = true,
    [18244] = true,
    [18243] = true,
    [6898] = true,
    [18246] = true,
    [6627] = true,
    [15855] = true,
    [9221] = true,
    [18502] = true,
    [13380] = true,
    [18630] = true,
    [22789] = true,
    [3370] = true,
    [6755] = true,
    [4979] = true,
    [12065] = true,
    [885] = true,
    [3562] = true,
    [9573] = true,
    [13700] = true,
    [3566] = true,
    [23429] = true,
    [6899] = true,
    [20380] = true,
    [13860] = true,
    [7322] = true,
    [8211] = true,
    [5187] = true,
    [3498] = true,
    [19846] = true,
    [9957] = true,
    [19974] = true,
    [5219] = true,
    [10053] = true,
    [8435] = true,
    [3722] = true,
    [10149] = true,
    [10181] = true,
    [28612] = true,
    [3586] = true,
    [28740] = true,
    [1050] = true,
    [20678] = true,
    [16647] = true,
    [20806] = true,
    [14532] = true,
    [928] = true,
    [3650] = true,
    [16967] = true,
    [15264] = true,
    [2539] = true,
    [1042] = true,
    [3690] = true,
    [17287] = true,
    [3866] = true,
    [3870] = true,
    [10789] = true,
    [3561] = true,
    [7794] = true,
    [3064] = true,
    [10917] = true,
    [982] = true,
    [17639] = true,
    [9795] = true,
    [597] = true,
    [1004] = true,
    [9939] = true,
    [26373] = true,
    [18247] = true,
    [12098] = true,
    [4066] = true,
    [3842] = true,
    [3850] = true,
    [3858] = true,
    [11365] = true,
    [11397] = true,
    [17463] = true,
    [7779] = true,
    [7795] = true,
    [17458] = true,
    [7827] = true,
    [17456] = true,
    [7859] = true,
    [17455] = true,
    [15780] = true,
    [23430] = true,
    [27589] = true,
    [19463] = true,
    [6499] = true,
    [19903] = true,
    [15972] = true,
    [17290] = true,
    [15575] = true,
    [19847] = true,
    [24006] = true,
    [19975] = true,
    [12069] = true,
    [28293] = true,
    [17204] = true,
    [8131] = true,
    [20295] = true,
    [13378] = true,
    [17181] = true,
    [12824] = true,
    [16997] = true,
    [4132] = true,
    [20297] = true,
    [4164] = true,
    [20807] = true,
    [24966] = true,
    [8422] = true,
    [12549] = true,
    [3515] = true,
    [19886] = true,
    [2671] = true,
    [2675] = true,
    [7443] = true,
    [7451] = true,
    [16788] = true,
    [7483] = true,
    [16783] = true,
    [8774] = true,
    [8806] = true,
    [1366] = true,
    [30021] = true,
    [16726] = true,
    [26054] = true,
    [17928] = true,
    [7418] = true,
    [16654] = true,
    [16653] = true,
    [1464] = true,
    [26438] = true,
    [16648] = true,
    [18376] = true,
    [9222] = true,
    [16644] = true,
    [11988] = true,
    [3015] = true,
    [16599] = true,
    [10165] = true,
    [16598] = true,
    [1538] = true,
    [13626] = true,
    [12826] = true,
    [13637] = true,
    [9574] = true,
    [16533] = true,
    [3211] = true,
    [23431] = true,
    [27590] = true,
    [9734] = true,
    [6500] = true,
    [13140] = true,
    [3295] = true,
    [13220] = true,
    [19784] = true,
    [9926] = true,
    [4980] = true,
    [3319] = true,
    [16055] = true,
    [10054] = true,
    [3359] = true,
    [3363] = true,
    [10150] = true,
    [3371] = true,
    [16741] = true,
    [3611] = true,
    [1698] = true,
    [3399] = true,
    [20680] = true,
    [16649] = true,
    [20808] = true,
    [20872] = true,
    [25031] = true,
    [5252] = true,
    [5268] = true,
    [7124] = true,
    [3071] = true,
    [25351] = true,
    [8407] = true,
    [7437] = true,
    [1980] = true,
    [3651] = true,
    [17481] = true,
    [5269] = true,
    [10854] = true,
    [15861] = true,
    [21832] = true,
    [10790] = true,
    [21960] = true,
    [16396] = true,
    [22088] = true,
    [8778] = true,
    [3815] = true,
    [18406] = true,
    [26439] = true,
    [3871] = true,
    [7748] = true,
    [18441] = true,
    [15596] = true,
    [5668] = true,
    [11366] = true,
    [15493] = true,
    [15495] = true,
    [2008] = true,
    [6741] = true,
    [8092] = true,
    [15653] = true,
    [10151] = true,
    [23240] = true,
    [15050] = true,
    [15781] = true,
    [23432] = true,
    [14887] = true,
    [14871] = true,
    [14810] = true,
    [6501] = true,
    [15973] = true,
    [14809] = true,
    [14200] = true,
    [19849] = true,
    [16101] = true,
    [13947] = true,
    [12070] = true,
    [13912] = true,
    [2392] = true,
    [7054] = true,
    [12198] = true,
    [13898] = true,
    [2480] = true,
    [13663] = true,
    [7213] = true,
    [13655] = true,
    [6196] = true,
    [16650] = true,
    [7389] = true,
    [20873] = true,
    [8423] = true,
    [9918] = true,
    [16970] = true,
    [18442] = true,
    [2840] = true,
    [17162] = true,
    [12524] = true,
    [12742] = true,
    [8679] = true,
    [12421] = true,
    [9945] = true,
    [3016] = true,
    [12199] = true,
    [9820] = true,
    [17738] = true,
    [8903] = true,
    [26056] = true,
    [3116] = true,
    [8394] = true,
    [12080] = true,
    [3204] = true,
    [9095] = true,
    [26440] = true,
    [8986] = true,
    [12058] = true,
    [9223] = true,
    [12048] = true,
    [6692] = true,
    [11963] = true,
    [6766] = true,
    [3396] = true,
    [6918] = true,
    [3488] = true,
    [3500] = true,
    [3508] = true,
    [11993] = true,
    [9575] = true,
    [13702] = true,
    [7078] = true,
    [7222] = true,
    [8331] = true,
    [6654] = true,
    [27720] = true,
    [10960] = true,
    [8795] = true,
    [8939] = true,
    [10788] = true,
    [7766] = true,
    [9959] = true,
    [15800] = true,
    [24137] = true,
    [7076] = true,
    [10216] = true,
    [6639] = true,
    [20298] = true,
    [2637] = true,
    [15066] = true,
}
